/*
**    PROJET 3ETI CPE Lyon
**    Copyright (C) 2011 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _COLOR_RGB_HPP_
#define _COLOR_RGB_HPP_

#include <iostream>
#include <exception_cpe.hpp>


namespace cpe
{
    /** \brief Container class for (r,g,b) color. Each component is an integer in [0,255] */
    class color
    {
    public:


        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor (0,0,0) */
        color();
        /** \brief direct constructor with (r,b,g) value */
        color(const unsigned char& r,const unsigned char& g,const unsigned char& b);

        // ********************************************* //
        // ********************************************* //
        //  ACCESS COLOR VALUE
        // ********************************************* //
        // ********************************************* //

        /** \brief get red value */
        const unsigned char& r() const;
        /** \brief get red value */
        unsigned char& r();

        /** \brief get green value */
        const unsigned char& g() const;
        /** \brief get green value */
        unsigned char& g();

        /** \brief get blue value */
        const unsigned char& b() const;
        /** \brief get blue value */
        unsigned char& b();


        // ********************************************* //
        // ********************************************* //
        //  I/O
        // ********************************************* //
        // ********************************************* //

        /** \brief standard output */
        friend std::ostream& operator<<(std::ostream& stream,const color& c);


    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL STORAGE
        // ********************************************* //
        // ********************************************* //

        /** \brief internal red component */
        unsigned char internal_r;
        /** \brief internal green component */
        unsigned char internal_g;
        /** \brief internal blue component */
        unsigned char internal_b;

    };


}

#endif
