/*
**    PROJET 3ETI CPE Lyon
**    Copyright (C) 2011 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef _IMAGE_HPP_
#define _IMAGE_HPP_

#include <vector>
#include <color.hpp>

namespace cpe
{

    /** \brief Container for picture data. Data are stored as unsigned char [0,255] */
    class image
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor */
        image();
        /** \brief create picture of size Nx x Ny
            \param unsigned int Nx: picture width
            \param unsigned int Ny: picture height
        */
        image(const unsigned int& Nx,const unsigned int& Ny);

        // ********************************************* //
        // ********************************************* //
        //  Picture Size
        // ********************************************* //
        // ********************************************* //

        /** \brief get the picture width in pixel */
        const unsigned int& Nx() const;
        /** \brief get the picture height in pixel */
        const unsigned int& Ny() const;

        // ********************************************* //
        // ********************************************* //
        //  Pixel access
        // ********************************************* //
        // ********************************************* //

        /** \brief get pixel (px,py) */
        const color& operator()(const int& px,const int& py) const;
        /** \brief get pixel (px,py) */
        color& operator()(const int& px,const int& py);



        // ********************************************* //
        // ********************************************* //
        //  I/O
        // ********************************************* //
        // ********************************************* //

        /** \brief read a ppm picture format (ascii)
            \param std::string filename: path of the file to read
        */
        void read_ppm(const std::string& filename);
        /** \brief Export the picture into ppm format (ascii)
            \param std::string filename: file to store the picture
        */
        void export_ppm(const std::string& filename);

    private:


        // ********************************************* //
        // ********************************************* //
        //  Internal data
        // ********************************************* //
        // ********************************************* //

        /** \brief storage of picture width */
        unsigned int internal_Nx;
        /** \brief storage of picture height */
        unsigned int internal_Ny;

        /** \brief internal data storage as std::vector of unsigned char

            Picture is stored as (r0,b0,g0,a0 , r1,b1,g1,a1 , ...) vector size is therefore 4 Nx Ny
        */
        std::vector<color> internal_data;


        // ********************************************* //
        // ********************************************* //
        //  Internal Methods
        // ********************************************* //
        // ********************************************* //

        /** \brief check if a given pixel (kx,ky) is within the size of the picture
            \return true if (kx,ky) is within the picture, false otherwise
        */
        bool check_position(const int& px,const int& py) const;

    };

}

#endif

