/*
**    PROJET 3ETI CPE Lyon
**    Copyright (C) 2011 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <exception_cpe.hpp>

#include <image.hpp>
#include <string>
#include <sstream>
#include <fstream>

namespace cpe
{

    image::image():internal_Nx(0),internal_Ny(0){}
    image::image(const unsigned int& Nx,const unsigned int& Ny):internal_Nx(Nx),internal_Ny(Ny){internal_data.resize(internal_Nx*internal_Ny);}

    const unsigned int& image::Nx() const{return internal_Nx;}
    const unsigned int& image::Ny() const{return internal_Ny;}

    bool image::check_position(const int& px,const int& py) const
    {
        return !(px<0 || py<0 || px>=static_cast<int>(internal_Nx) || py>=static_cast<int>(internal_Ny));
    }

    const color& image::operator()(const int& px,const int& py) const
    {
        if(check_position(px,py)==true)
            return internal_data[px+internal_Nx*py];

        std::ostringstream msg;msg<<"(px,py)=("<<px<<","<<py<<") size=("<<internal_Nx<<","<<internal_Ny<<")";
        throw exception_cpe(msg.str(),__FILE__,__FUNCTION__,__LINE__);
    }

    color& image::operator()(const int& px,const int& py)
    {
        if(check_position(px,py)==true)
            return internal_data[px+internal_Nx*py];

        std::ostringstream msg;msg<<"(px,py)=("<<px<<","<<py<<") size=("<<internal_Nx<<","<<internal_Ny<<")";
        throw exception_cpe(msg.str(),__FILE__,__FUNCTION__,__LINE__);
    }


    void image::read_ppm(const std::string& filename)
    {
        std::ifstream stream(filename.c_str());
        if(!stream.good())
            throw exception_cpe("Error in opening file ["+filename+"] ... : cannot open file ",__FILE__,__FUNCTION__,__LINE__);

        //go to magic number
        std::string buffer;
        buffer="#";
        while(buffer[0]=='#')
            std::getline(stream,buffer);
        if(buffer!="P3")
            throw exception_cpe("Cannot find P3 magic number in file "+filename,__FILE__,__FUNCTION__,__LINE__);

        //(Nx,Ny)
        buffer="#";
        while(buffer[0]=='#')
            std::getline(stream,buffer);
        std::istringstream tokens(buffer);

        unsigned int temp_Nx=0,temp_Ny=0;
        tokens>>temp_Nx;tokens>>temp_Ny;

        internal_Nx=temp_Nx;
        internal_Ny=temp_Ny;

        //color number
        stream>>buffer;


        unsigned int red,green,blue;

        internal_data.resize(temp_Nx*temp_Ny);

        for(unsigned int k_y=0;k_y<temp_Ny;k_y++)
        {
            for(unsigned int k_x=0;k_x<temp_Nx;k_x++)
            {
                stream>>red;stream>>green;stream>>blue;
                internal_data[temp_Nx*k_y+k_x]=color(red,green,blue);
            }
        }


        stream.close();
    }

    void image::export_ppm(const std::string& filename)
    {
        FILE *fid=fopen(filename.c_str(),"w");

        fprintf(fid,"P3\n%d %d\n255\n",internal_Nx,internal_Ny);



        for(unsigned int k2=0;k2<internal_Ny;++k2)
        {
            for(unsigned int k1=0;k1<internal_Nx;++k1)
            {
                fprintf(fid,"%u %u %u\n",
                        static_cast<unsigned int>(internal_data[internal_Nx*k2+k1].r()),
                        static_cast<unsigned int>(internal_data[internal_Nx*k2+k1].g()),
                        static_cast<unsigned int>(internal_data[internal_Nx*k2+k1].b()));
            }
        }
        fclose(fid);
    }




}



